const path = require('path');
const webpack = require('webpack');
const webpackMerge = require('webpack-merge');
const commonConfig = require('./webpack.common.js');
const themes = require('../src/themes/themes.json');

// webpack plugins
const UglifyJsPlugin = require('webpack/lib/optimize/UglifyJsPlugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CommonsChunkPlugin = webpack.optimize.CommonsChunkPlugin;

const entries = {qgrid: './src/build.js'};

themes.forEach(theme => {
	entries[`qgrid.${theme.path}`] = path.join(__dirname, '..', 'src', 'themes', theme.path)
});

module.exports = webpackMerge(commonConfig, {
	entry: entries,
	output: {
		path: path.join(__dirname, '..', 'dist'),
		filename: '[name].min.js',
		sourceMapFilename: '[name].min.map',
		library: 'qgrid'
	},

	/**
	 * Add additional plugins to the compiler.
	 *
	 * See: http://webpack.github.io/docs/configuration.html#plugins
	 */
	plugins: [
		new webpack.LoaderOptionsPlugin({
			options: {
				eslint: {
					failOnWarning: false,
					failOnError: true
				}
			}
		}),
		new ExtractTextPlugin({
			filename: '[name].min.css',
			disable: false,
			allChunks: true
		}),

		new UglifyJsPlugin({
			sourceMap: true,
			minimize: true, //prod
			beautify: false, //prod
			mangle: {screw_ie8: true}, //prod
			compress: {screw_ie8: true, warnings: true, }, //prod
			comments: false, //prod
			pure_funcs: ['Log.info', 'Log.error', 'Log.warning']
		})
	]
});